/*   AM-RGB for ICP-PCI Kernel module 
 *   free public release
 *
 *   Modified by: Rin. Taniguchi    email: rin@is.kyushu-u.ac.jp
 *                Laboratory for Image and Media Understanding
 *                Dept. of Intelligent Systems
 *                Kyushu University, Japan
 *
 *      referring to: AM-CLR developed by the following people
 *
 *   Werner Jaeger        email: e9026603@student.tuwien.ac.at
 *   Wilfried Kubinger    email: wk@flexaut.tuwien.ac.at
 *
 *   Institute of Flexible Automation
 *   University of Technology Vienna
 *   Austria, Europe
 *
 *   amrgbReg.c  v1.0  1998/01/05
 */

/* Change History:
 
   - 1999/05/03 13:20:45 matthias
     ajustments to version 0.5.0 (some datatypes and variables renamed)
   
 */

/*
 * Defines:
 */

/*
 * Includes:
 */

#include "icpciOS.h"	/* os specific stuff */
#include "icpciExt.h"	/* offical driver interface */
#include "icpciInt.h"	/* driver internals */
#include "amrgbReg.h"	/* register definitions AM-RGB */

/*
 * Globals:
 */

extern struct
icp_global_t icp;	/* main driver structure */

#ifdef DEBUG_ICPCI_REG
const char *rgb_reg[] = 
{ 
  "modid   " , "fpga    " , "error   " , "camcon  " , "modcon  " ,
  "error   " , "error   " , "error   " , "error   " , "error   " ,
  "error   " , "error   " , "cscadrlo" , "cscadrhi" , "cscdat  " ,
  "error   " , "adccon  " , "rnref   " , "rpref   " , "gnref   " , 
  "gpref   " , "bnref   " , "bpref   " , "error   " , "alutadr " ,
  "blutadr " , "clutadr " , "alutdat " , "blutdat " , "clutdat " ,
  "error   " , "error   "  
};
#endif /* DEBUG_ICPCI_REG */

/*
 * Prototypes:
 */

/*
 * General Read Handler for AM-CLR Registers
 */

static u_short
rgb_read_mm (board, offset)

  u_char  board;         /* number of IC-PCI board attached to the computer */
  u_char  offset;        /* offset to register base address pointer */

{
  u_short tmp_val = 0;
  u_short amcc_dummy = 0;

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board, icp.map[board]->v_cr_base + offset / 2,
			ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset / 2);
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }
  tmp_val = icp_readw (icp.map[board]->v_mod_base + offset);

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  rgb-readw  %s, %#06X\n",
	     board, rgb_reg[offset], tmp_val);
#endif

  return (tmp_val);
}

/* 16 bit register 1 (or more) bit access */

static u_short 
rgb_read_ms (board, offset, mask, shift)

  u_char  board;         /* number of IC-PCI board attached to the computer */
  u_char  offset;        /* offset to register base address pointer */
  u_short  mask;          /* read mask for register */
  u_char  shift;         /* number of bits to shift right */  

{
  u_short tmp_val = 0;
  u_short amcc_dummy = 0;

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board,icp.map[board]->v_cr_base + offset / 2,
			ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset / 2);
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }
  tmp_val = icp_readw (icp.map[board]->v_mod_base + offset);

  tmp_val &= mask;
  tmp_val >>= shift;

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  rgb-readw  %s, %#1X &(%#06X) >>%2d\n",
	     board, rgb_reg[offset], tmp_val, mask, shift);
#endif

  return (tmp_val);
}
  

/*
 * General Write Handler for AM-CLR Registers
 */

/* 16 bit register access */

static void 
rgb_write_mm (board, offset, value)

  u_char  board;         /* number of IC-PCI board attached to the computer */
  u_char  offset;        /* offset to register base address pointer */
  u_short  value;         /* value to set to */  

{
  u_short tmp_val = 0;
  u_short amcc_dummy = 0;

  tmp_val = value;

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  rgb-writew %s, %#06X\n",
	     board, rgb_reg[offset], tmp_val);
#endif

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board,icp.map[board]->v_cr_base + offset / 2,
			ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset / 2);
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }
  icp_writew (tmp_val, icp.map[board]->v_mod_base + offset);
}

/*
 * Special Read Handler for AM-CLR Registers
 */

/* --- Module-ID Register --- */

u_short 
rgb_get_rev (u_char board) {

  return (rgb_read_ms (board, 
                      RGB_MODID_OFFSET, 
                      RGB_REV_MASK,
                      RGB_REV_SHIFT));
}

u_short 
rgb_get_type (u_char board) {

  return (rgb_read_ms (board, 
                      RGB_MODID_OFFSET, 
                      RGB_TYPE_MASK,
                      RGB_TYPE_SHIFT));
}

/* --- AM RGB Register Initialize --- */

/* --- write timing into FPGA-Chip --- */

int 
rgb_set_fpga (u_char board, struct fpga_buffer_t *fpga) {

  int i, count;

  rgb_write_mm(board, RGB_FPGA_CLEAR, 0); /* Reset ORCA */

  icp_udelay(1000);
  count = 0;

  while (count<fpga->size) {
    /* Write decimal value to the AMS. */
    for (i=0; i<16; i++) {
      if (count>=fpga->size) break;
      rgb_write_mm(board, RGB_FPGA_OFFSET,
		   ((fpga->buffer[count++] & 0xff) | 0x200));
      rgb_read_mm(board, RGB_MODID_OFFSET);
      rgb_read_mm(board, RGB_MODID_OFFSET);
      rgb_read_mm(board, RGB_MODID_OFFSET);
    }		
    icp_udelay(1000);
  }
  
  rgb_write_mm(board, RGB_FPGA_OFFSET,0x0000);
  rgb_write_mm(board, RGB_FPGA_OFFSET,0x00ff);
 
  icp_udelay(1000);

  return (OK);       
}

void amrgb_default_setting(struct amrgb_cam_t *camcnf)
{
  camcnf->width = RGB_IMAGE_WIDTH_DEFAULT;
  camcnf->height = RGB_IMAGE_HEIGHT_DEFAULT;
  camcnf->channel = 0;
  camcnf->field = 0;
  /* for register setting */
  camcnf->mxctl = (RGB_MUXSEL0_DEFAULT<<RGB_MUXSEL0_SHIFT) |
    (RGB_MUXSEL1_DEFAULT<<RGB_MUXSEL1_SHIFT) |
    (RGB_MUXSEL2_DEFAULT<<RGB_MUXSEL2_SHIFT) |
    (RGB_SDMODE_DEFAULT<<RGB_SDMODE_SHIFT) ;
  camcnf->htotal = RGB_HTOTAL_DEFAULT;
  camcnf->hesync = RGB_HESYNC_DEFAULT;
  camcnf->heeq = RGB_HEEQ_DEFAULT;
  camcnf->serend = RGB_SEREND_DEFAULT;
  camcnf->vtotal = RGB_VTOTAL_DEFAULT;
  camcnf->vsync = (RGB_VSEND_DEFAULT<<RGB_VSEND_SHIFT) |
    (RGB_PLLCEN_DEFAULT<<RGB_PLLCEN_SHIFT);
  camcnf->vsgtrt = RGB_VSGTRT_DEFAULT;
  camcnf->vgend = RGB_VGEND_DEFAULT;
  camcnf->hoff = RGB_HOFF_DEFAULT;
  camcnf->hactive = RGB_HACTIVE_DEFAULT;
  camcnf->voff = RGB_VOFF_DEFAULT;
  camcnf->vactive = RGB_VACTIVE_DEFAULT;
  camcnf->strbdly = RGB_STRBDLY_DEFAULT;
  camcnf->freset = (RGB_FROFF_DEFAULT<<RGB_FROFF_SHIFT) |
    (RGB_FRSTMD_DEFAULT<<RGB_FRSTMD_SHIFT) |
    (RGB_FRSTPOL_DEFAULT<<RGB_FRSTPOL_SHIFT) |
    (RGB_FRSTSZ_DEFAULT<<RGB_FRSTSZ_SHIFT)	       ;
  camcnf->con1 = (RGB_FENPOL_DEFAULT<<RGB_FENPOL_SHIFT) |
    (RGB_LENPOL_DEFAULT<<RGB_LENPOL_SHIFT) |
    (RGB_FLDPOL_DEFAULT<<RGB_FLDPOL_SHIFT) |
    (RGB_FLDSEL_DEFAULT<<RGB_FLDSEL_SHIFT) |
    (RGB_VSCAN_DEFAULT<<RGB_VSCAN_SHIFT) |
    (RGB_TIMEMD_DEFAULT<<RGB_TIMEMD_SHIFT) |
    (RGB_VSYNCEN_DEFAULT<<RGB_VSYNCEN_SHIFT) |
    (RGB_SYNCSEL_DEFAULT<<RGB_SYNCSEL_SHIFT) |
    (RGB_FLDSHFT_DEFAULT<<RGB_FLDSHFT_SHIFT);
  camcnf->rgbtrig = (RGB_TRIGEN0_DEFAULT<<RGB_TRIGEN0_SHIFT) |
    (RGB_TRIGEN1_DEFAULT<<RGB_TRIGEN1_SHIFT) |
    (RGB_TRIGEN2_DEFAULT<<RGB_TRIGEN2_SHIFT) |
    (RGB_TRGSRC_DEFAULT<<RGB_TRGSRC_SHIFT) |
    (RGB_TRGPOL_DEFAULT<<RGB_TRGPOL_SHIFT) |
    (RGB_TRIGLACE_DEFAULT<<RGB_TRIGLACE_SHIFT) |
    (RGB_STRBMD_DEFAULT<<RGB_STRBMD_SHIFT) |
    (RGB_STRBPOL_DEFAULT<<RGB_STRBPOL_SHIFT) |
    (RGB_SKPFLDMD_DEFAULT<<RGB_SKPFLDMD_SHIFT);
  camcnf->rgbswtrig = (RGB_SWTRIG0_DEFAULT<<RGB_SWTRIG0_SHIFT) |
    (RGB_SWTRIG1_DEFAULT<<RGB_SWTRIG1_SHIFT) |
    (RGB_SWTRIG2_DEFAULT<<RGB_SWTRIG2_SHIFT);
  camcnf->pclamp = (RGB_CCNT_DEFAULT<<RGB_CCNT_SHIFT) |
    (RGB_PCLMPOL_DEFAULT<<RGB_PCLMPOL_SHIFT) |
    (RGB_CLMPSRC_DEFAULT<<RGB_CLMPSRC_SHIFT);
  camcnf->miscctl = (RGB_AMSMODE_DEFAULT<<RGB_AMSMODE_SHIFT) |
    (RGB_VCLKPOL_DEFAULT<<RGB_VCLKPOL_SHIFT) |
    (RGB_PLLCS_DEFAULT<<RGB_PLLCS_SHIFT) |
    (RGB_DACCS_DEFAULT<<RGB_DACCS_SHIFT) |
    (RGB_DACRDBK_DEFAULT<<RGB_DACRDBK_SHIFT) |
    (RGB_LPFSEL_DEFAULT<<RGB_LPFSEL_SHIFT) |
    (RGB_LUTOSEL_DEFAULT<<RGB_LUTOSEL_SHIFT) |
    (RGB_VIDINSEL_DEFAULT<<RGB_VIDINSEL_SHIFT) |
    (RGB_PATHSEL_DEFAULT<<RGB_PATHSEL_SHIFT) |
    (RGB_INTSEL_DEFAULT<<RGB_INTSEL_SHIFT) |
    (RGB_STRPSRC_DEFAULT<<RGB_STRPSRC_SHIFT);
  camcnf->rgbgain = (RGB_REDG_DEFAULT<<RGB_REDG_SHIFT) |
    (RGB_GRNG_DEFAULT<<RGB_GRNG_SHIFT) |
    (RGB_BLUG_DEFAULT<<RGB_BLUG_SHIFT);
  /* for PLL */
  camcnf->fdiv = PLL_FDIV_DEFAULT;
  camcnf->lo = PLL_LO_DEFAULT;
  camcnf->hi = PLL_HI_DEFAULT;
  camcnf->rdiv = PLL_RDIV_DEFAULT;
  camcnf->refpol = PLL_REFPOL_DEFAULT;
  camcnf->vco = PLL_VCO_DEFAULT;
  camcnf->pfd = PLL_PFD_DEFAULT;
  camcnf->pden = PLL_PDEN_DEFAULT;
  camcnf->intflt = PLL_INTFLT_DEFAULT;
  camcnf->intvco = PLL_INTVCO_DEFAULT;
  camcnf->clksel = PLL_CLKSEL_DEFAULT;
  camcnf->aux4 = PLL_AUX4_DEFAULT;
  camcnf->fbksel = PLL_FBKSEL_DEFAULT;
  camcnf->fbkpol = PLL_FBKPOL_DEFAULT;
  camcnf->pda = PLL_PDA_DEFAULT;
  camcnf->pdb = PLL_PDB_DEFAULT;
  camcnf->ldlg = PLL_LDLG_DEFAULT;
  camcnf->fineen = PLL_FINEEN_DEFAULT;
  camcnf->aux5 = PLL_AUX5_DEFAULT;
  camcnf->lcnt = PLL_LCNT_DEFAULT;
  camcnf->omux1 = PLL_OMUX1_DEFAULT;
  camcnf->extref = PLL_EXTREF_DEFAULT;
  /* for DAC */
  camcnf->rnref = RGB_RNREF_DEFAULT;
  camcnf->rpref = RGB_RPREF_DEFAULT;
  camcnf->gnref = RGB_GNREF_DEFAULT;
  camcnf->gpref = RGB_GPREF_DEFAULT;
  camcnf->bnref = RGB_BNREF_DEFAULT;
  camcnf->bpref = RGB_BPREF_DEFAULT;
  camcnf->clamp_volt = RGB_CLAMP_VOLT;
}

static void rgb_init_pll(u_char);
static void rgb_init_dac(u_char);

void
rgb_init_regs (u_char board)
{
  rgb_write_mm(board, RGB_MXCTL_OFFSET,
	       icp.dev[board][0]->amrgb_cam.mxctl);
  rgb_write_mm(board, RGB_HTOTAL_OFFSET,
	       icp.dev[board][0]->amrgb_cam.htotal);
  rgb_write_mm(board, RGB_HESYNC_OFFSET,
	       icp.dev[board][0]->amrgb_cam.hesync);
  rgb_write_mm(board, RGB_HEEQ_OFFSET,
	       icp.dev[board][0]->amrgb_cam.heeq);
  rgb_write_mm(board, RGB_SEREND_OFFSET,
	       icp.dev[board][0]->amrgb_cam.serend);
  rgb_write_mm(board, RGB_VTOTAL_OFFSET,
	       icp.dev[board][0]->amrgb_cam.vtotal);
  rgb_write_mm(board, RGB_VSYNC_OFFSET,
	       icp.dev[board][0]->amrgb_cam.vsync);
  rgb_write_mm(board, RGB_VSGTRT_OFFSET,
	       icp.dev[board][0]->amrgb_cam.vsgtrt);
  rgb_write_mm(board, RGB_VGEND_OFFSET,
	       icp.dev[board][0]->amrgb_cam.vgend);
  rgb_write_mm(board, RGB_HOFF_OFFSET,
	       icp.dev[board][0]->amrgb_cam.hoff);
  rgb_write_mm(board, RGB_HACTIVE_OFFSET,
	       icp.dev[board][0]->amrgb_cam.hactive);
  rgb_write_mm(board, RGB_VOFF_OFFSET,
	       icp.dev[board][0]->amrgb_cam.voff);
  rgb_write_mm(board, RGB_VACTIVE_OFFSET,
	       icp.dev[board][0]->amrgb_cam.vactive);
  rgb_write_mm(board, RGB_STRBDLY_OFFSET,
	       icp.dev[board][0]->amrgb_cam.strbdly);
  rgb_write_mm(board, RGB_FRESET_OFFSET,
	       icp.dev[board][0]->amrgb_cam.freset);
  rgb_write_mm(board, RGB_CON1_OFFSET,
	       icp.dev[board][0]->amrgb_cam.con1);
  rgb_write_mm(board, RGB_RGBTRIG_OFFSET,
	       icp.dev[board][0]->amrgb_cam.rgbtrig);
  rgb_write_mm(board, RGB_RGBSWTRIG_OFFSET,
	       icp.dev[board][0]->amrgb_cam.rgbswtrig);
  rgb_write_mm(board, RGB_PCLAMP_OFFSET,
	       icp.dev[board][0]->amrgb_cam.pclamp);
  rgb_write_mm(board, RGB_MISCCTL_OFFSET,
	       icp.dev[board][0]->amrgb_cam.miscctl);
  rgb_write_mm(board, RGB_RGBGAIN_OFFSET,
	       icp.dev[board][0]->amrgb_cam.rgbgain);

  rgb_init_dac(board);
  rgb_init_pll(board);

}

static void rgb_write_dac(u_char, unsigned int);

static
void
rgb_init_dac(u_char board)
{
  rgb_write_dac(board, (3<<20) | (icp.dev[board][0]->amrgb_cam.rnref<<12) |
			(3<<8) | icp.dev[board][0]->amrgb_cam.rpref);
  rgb_write_dac(board, (7<<20) | (icp.dev[board][0]->amrgb_cam.gnref<<12) |
			(7<<8) | icp.dev[board][0]->amrgb_cam.gpref);
  rgb_write_dac(board, (0xb<<20) | (icp.dev[board][0]->amrgb_cam.bnref<<12) |
			(0xb<<8) | icp.dev[board][0]->amrgb_cam.bpref);
  rgb_write_dac(board, (0xf<<20) | (icp.dev[board][0]->amrgb_cam.clamp_volt<<12));
}

static
void
rgb_write_dac (u_char board, unsigned int value)
{
  u_short tmp;
  int i;

  tmp = rgb_read_mm (board, RGB_MISCCTL_OFFSET);
  rgb_write_mm (board, RGB_MISCCTL_OFFSET, tmp & (~RGB_DACCS_MASK));

  for (i=23; i>=0; i--) {
    rgb_write_mm (board, RGB_DACPRG_OFFSET, (value >> i) & 1);
  }

  rgb_write_mm (board, RGB_MISCCTL_OFFSET, tmp | RGB_DACCS_MASK);

}

static
void
rgb_write_pll (u_char board, u_char offset, u_short value)
{
  u_short tmp;
  int i;

  tmp = rgb_read_mm (board, RGB_MISCCTL_OFFSET);
  rgb_write_mm (board, RGB_MISCCTL_OFFSET, tmp & (~RGB_PLLCS_MASK));

  rgb_write_mm (board, RGB_PLLRPG_OFFSET, 0);
  for (i=0; i<3; i++) {
    rgb_write_mm (board, RGB_PLLRPG_OFFSET, offset & 1);
    offset >>= 1;
  }
  for (i=0; i<11; i++) {
    rgb_write_mm (board, RGB_PLLRPG_OFFSET, value & 1);
    value >>= 1;
  }
  rgb_write_mm (board, RGB_MISCCTL_OFFSET, tmp | RGB_PLLCS_MASK);

  rgb_write_mm (board, RGB_PLLRPG_OFFSET, 0);
  
}

static
void
rgb_init_pll(u_char board)
{
  rgb_write_pll(board, PLL_REG0_OFFSET,
		icp.dev[board][0]->amrgb_cam.fdiv&PLL_FDIV_MASK);
  rgb_write_pll(board, PLL_REG1_OFFSET,
		icp.dev[board][0]->amrgb_cam.lo&PLL_LO_MASK);
  rgb_write_pll(board, PLL_REG2_OFFSET,
		icp.dev[board][0]->amrgb_cam.hi&PLL_HI_MASK);
  rgb_write_pll(board, PLL_REG3_OFFSET,
		(icp.dev[board][0]->amrgb_cam.rdiv<<PLL_RDIV_SHIFT) |
		(icp.dev[board][0]->amrgb_cam.refpol<<PLL_REFPOL_SHIFT));
  rgb_write_pll(board, PLL_REG4_OFFSET,
		(icp.dev[board][0]->amrgb_cam.vco<<PLL_VCO_SHIFT) |
		(icp.dev[board][0]->amrgb_cam.pfd<<PLL_PFD_SHIFT) |
		(icp.dev[board][0]->amrgb_cam.pden<<PLL_PDEN_SHIFT) |
		(icp.dev[board][0]->amrgb_cam.intflt<<PLL_INTFLT_SHIFT) |
		(icp.dev[board][0]->amrgb_cam.intvco<<PLL_INTVCO_SHIFT) |
		(icp.dev[board][0]->amrgb_cam.clksel<<PLL_CLKSEL_SHIFT) |
		(icp.dev[board][0]->amrgb_cam.aux4<<PLL_AUX4_SHIFT));
  rgb_write_pll(board, PLL_REG5_OFFSET,
		(icp.dev[board][0]->amrgb_cam.fbksel<<PLL_FBKSEL_SHIFT) |
		(icp.dev[board][0]->amrgb_cam.fbkpol<<PLL_FBKPOL_SHIFT) |
		(icp.dev[board][0]->amrgb_cam.pda<<PLL_PDA_SHIFT) |
		(icp.dev[board][0]->amrgb_cam.pdb<<PLL_PDB_SHIFT) |
		(icp.dev[board][0]->amrgb_cam.ldlg<<PLL_LDLG_SHIFT) |
		(icp.dev[board][0]->amrgb_cam.fineen<<PLL_FINEEN_SHIFT) |
		(icp.dev[board][0]->amrgb_cam.aux5<<PLL_AUX5_SHIFT));
  rgb_write_pll(board, PLL_REG6_OFFSET,
		(icp.dev[board][0]->amrgb_cam.lcnt<<PLL_LCNT_SHIFT) |
		(icp.dev[board][0]->amrgb_cam.omux1<<PLL_OMUX1_SHIFT) |
		(icp.dev[board][0]->amrgb_cam.extref<<PLL_EXTREF_SHIFT));
}

